function main()
clc;
clf;
clear;
global Y Y0 D N m M dt t tMax
global c_damping c_spring c_f_gravity

% constants
global x_ y_ z_ s_ p_ v_ f_ r_
x_ = 1; y_ = 2; z_ = 3;
s_ = 4;  % initial configuration
p_ = 1;
v_ = 2;
f_ = 3;
dd_ = 5;

global R
R = 0.5;

%deprecated
Y = [inf inf inf inf];
Y0 = Y;

% config
m = 1;   % mass of one particle
M = m*20; % mass of deformer
D = 3;
N = 9;
tMax = 15;
dt = 0.1;

c_damping = 1.0;
c_spring  = 12.0;
c_f_gravity = [ 0 0 -0.3 ];

saveImages = 1;

% Setup
t = 0;
iimg = 0;

box = initial_box();
% sph = initial_sphere();
% solids = { box sph };
solids = { box };
solids = dup_initials( solids );

% ball = initial_sphere();
% translate( ball, [ 0 1 0 ] );
ball = initial_deformer();
ball = translate( ball, [ 0.45 0.45 1+(R*2) ] );

deformers = { ball };
deformers = dup_initials( deformers );

tic

while t < tMax
    hold off;
    clf;
    
    % Iterate solids
    for iY = 1:length(solids)
        Y = solids{iY};
        %         Y = global_forces( Y );
        
        Y = iterate( t, Y );
        solids{iY} = Y;
        
        plotstuff(Y);
        
    end
    
    solids = clear_forces( solids );
    
    [solids,deformers] = enforce_global_constraints( solids, deformers );
    
    % Iterate deformers
    for iY = 1:length(deformers)
        Y = deformers{iY};
        
        Y = iterate_deformer( t, Y );
        deformers{iY} = Y;
        
        plot_deformer( Y );
    end
    
    deformers = clear_forces( deformers );
    
    t = t + dt;
    
    drawnow;
    title( sprintf('k=%f, damp=%f, t=%f', c_spring, c_damping, t) );
    
    if saveImages
        f = getframe;
        imwrite(f.cdata, sprintf('images/image%d.png',iimg));
        iimg = iimg + 1;
    end
    
    
    fprintf('t=%f, real t=%f\n', t, toc);
end

end

function solids = dup_initials( solids )
global x_ y_ z_ s_ p_ v_ f_
for iY = 1:length(solids)
    Y = solids{ iY };
    Y(:,:,s_) = Y(:,:,p_);
    solids{ iY } = Y;
end
end

function Y = iterate( t,Y )
global dt
f1 = ode( t, Y );
f2 = ode( t+(dt/2), Y + (dt/2).*f1 );
f3 = ode( t+(dt/2), Y + (dt/2).*f2 );
f4 = ode( t+dt,     Y +  dt   .*f3 );
Y = Y + dt/6 .* (f1 + 2*f2 + 2*f3 + f4);

Y = enforce_constraints(Y);
end

function Y = iterate_deformer( t,Y )
global x_ y_ z_ s_ p_ v_ f_ r_
Y = iterate( t,Y );
Y(:,:,s_) = Y(:,:,p_);
end

function a = clear_forces( a )
global x_ y_ z_ s_ p_ v_ f_ r_
for i = 1:length(a)
    Y = a{i};
    Y(:,:,f_) = 0;
    a{i} = Y;
end
end